@@ -1,5 +1,5 @@
 #if	HAVE_CONFIG_H
-#include	"courier_lib_config.h"
+#include	"libs/courier_lib_config.h"
 #endif
 #include	"courier.h"
 #include	"rw.h"
