@@ -3,7 +3,7 @@
 ** See COPYING for distribution information.
 */
 
-#include	"courier_lib_config.h"
+#include	"libs/courier_lib_config.h"
 #include	"courier.h"
 #if	HAVE_SYS_STAT_H
 #include	<sys/stat.h>
