@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2014 Double Precision, Inc.
+** Copyright 1998 - 2018 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -32,6 +32,7 @@
 #include	"comsubmitclient.h"
 #include	"comreadtime.h"
 #include	"rfc822/rfc822.h"
+#include	"rfc2045/rfc2045.h"
 #include	"waitlib/waitlib.h"
 #include	"esmtpiov.h"
 #include	"esmtpconfig.h"
@@ -60,7 +61,7 @@ static char *input_line="";
 static void cancelsubmit();
 static time_t data_timeout;
 
-static const char *tcpremoteip, *tcpremotehost;
+static const char *tcpremoteip, *tcpremotehost, *tcpremoteport;
 
 #define	INIT_TEERGRUBE	8
 #define	MAX_TEERGRUBE	128
@@ -109,6 +110,8 @@ void iov_logerror(const char *q, const char *p)
 	clog_msg_start_info();
 	clog_msg_str("error,relay=");
 	clog_msg_str(tcpremoteip);
+	clog_msg_str(",port=");
+	clog_msg_str(tcpremoteport);
 	if (ident)
 	{
 		clog_msg_str(",ident=\"");
@@ -162,15 +165,19 @@ EHLO_EXDATA_EXTENSION
 EHLO_SECURITY_EXTENSION
 "250-PIPELINING\r\n"
 "250-8BITMIME\r\n"
+"250-SMTPUTF8\r\n"
 "250-SIZE\r\n"
 "250 DSN\r\n";
-const char *me=config_me();
+const char *me=config_me_ace();
 const char *p, *q;
 
 	if (helobuf[0] == 0)
 	{
 		char *p;
 
+		while (*heloname == ' ')
+		       ++heloname;
+
 		strncat(helobuf, heloname, sizeof(helobuf)-1);
 
 		for (p=helobuf; *p; p++)
@@ -258,7 +265,7 @@ int	rc;
 	a=rfc822a_alloc(t);
 	if (!a)	clog_msg_errno();
 
-	if (a->naddrs != 1 || a->addrs[0].tokens == 0)
+	if (a->naddrs < 1 || a->addrs[0].tokens == 0)
 	{
 		addiovec_error("502 EXPN syntax error.");
 		rfc822a_free(a);
@@ -458,6 +465,8 @@ const char *p;
 
 	clog_msg_str("error,relay=");
 	clog_msg_str(tcpremoteip);
+	clog_msg_str(",port=");
+	clog_msg_str(tcpremoteport);
 	if ((p=getenv("TCPREMOTEINFO")) != 0 && *p)
 	{
 	char	*q=strdup(p), *r;
@@ -536,12 +545,13 @@ const char *q=skipaddress(&p);
 
 static int domailfrom(const char *p, const char *q)
 {
-const char *r, *s;
-char	retformat=0;
-const char *envid=0;
-size_t	envidlen=0;
-char	*buf;
-int	rc;
+	const char *r, *s;
+	char	retformat=0;
+	const char *envid=0;
+	size_t	envidlen=0;
+	char	*buf;
+	int	rc;
+	int	smtputf8=0;
 
 	hasexdata=0;
 	hasverp=0;
@@ -634,6 +644,19 @@ int	rc;
 				l=n;
 			}
 		}
+		else if (s - r >= 8 &&
+#if HAVE_STRNCASECMP
+			 strncasecmp(r, "SMTPUTF8", 8)
+#else
+			 strnicmp(r, "SMTPUTF8", 8)
+#endif
+			 == 0)
+		{
+			if (s-r == 8 || r[8] == '=')
+			{
+				smtputf8=1;
+			}
+		}
 		r= s-1;
 	}
 
@@ -656,6 +679,10 @@ int	rc;
 	{
 		strcat(buf, "S{STARTTLS}");
 	}
+	if (smtputf8)
+	{
+		strcat(buf, "8");
+	}
 
 	if (envidlen)
 	{
@@ -753,8 +780,8 @@ static int rcpttolocal(const char *p, const char *r, const char *q)
 static int dorcptto2(const char *p, const char *q)
 {
 int	notifyn=0,notifyf=0,notifys=0,notifyd=0;
-const char *orcpt=0;
-int	orcptlen=0;
+char	*orcpt_utf8=0;
+size_t  orcpt_utf8_len=0;
 const char *r, *s;
 char	*t, *buf;
 const char *relayclient=getenv("RELAYCLIENT");
@@ -806,13 +833,24 @@ int	rc;
 #endif
 			== 0)
 		{
+			char *encoded_orcpt;
+
 			r += 6;
-			orcpt=r;
-			orcptlen=s-r;
+
+			encoded_orcpt=malloc(s-r+1);
+			if (!encoded_orcpt)
+				abort();
+			memcpy(encoded_orcpt, r, s-r);
+			encoded_orcpt[s-r]=0;
+			orcpt_utf8=rfc6533_decode(encoded_orcpt);
+			free(encoded_orcpt);
 		}
 	}
 
-	buf=courier_malloc(q-p + relayclientl + orcptlen + 10);
+	if (orcpt_utf8)
+		orcpt_utf8_len=strlen(orcpt_utf8);
+
+	buf=courier_malloc(q-p + relayclientl + orcpt_utf8_len + 10);
 	memcpy(buf, p, q-p);
 	t=buf + (q-p);
 	if (relayclientl)
@@ -832,9 +870,12 @@ int	rc;
 			*t++ = 'D';
 	}
 	*t++ = '\t';
-	if (orcptlen)
-		memcpy(t, orcpt, orcptlen);
-	t[orcptlen]=0;
+
+	if (orcpt_utf8_len)
+		memcpy(t, orcpt_utf8, orcpt_utf8_len);
+	t[orcpt_utf8_len]=0;
+	if (orcpt_utf8)
+		free(orcpt_utf8);
 
 	submit_write_message(buf);
 	free(buf);
@@ -1118,8 +1159,9 @@ int	authenticated=0;
 
 	tcpremoteip=getenv("TCPREMOTEIP");
 	tcpremotehost=getenv("TCPREMOTEHOST");
+	tcpremoteport=getenv("TCPREMOTEPORT");
 
-	if (!tcpremoteip)
+	if (!tcpremoteip || !tcpremoteport)
 	{
 		fprintf(stderr, "%s: don't know your IP address, no dice.\n",
 			argv[0]);
@@ -1137,6 +1179,8 @@ int	authenticated=0;
 	clog_msg_start_info();
 	clog_msg_str("started,ip=[");
 	clog_msg_str(tcpremoteip);
+	clog_msg_str("],port=[");
+	clog_msg_str(tcpremoteport);
 	clog_msg_str("]");
 	clog_msg_send();
 
@@ -1338,10 +1382,11 @@ int	authenticated=0;
 			}
 			else
 			{
-				int rc=auth_generic("esmtp", authtype,
-						    authdata,
-						    auth_callback_func,
-						    authmethod);
+				int rc=auth_generic_meta(NULL,
+							 "esmtp", authtype,
+							 authdata,
+							 auth_callback_func,
+							 authmethod);
 
 				if (rc)
 				{
