@@ -379,8 +379,7 @@ static void dofwd(const char *addr, const char *from, const char *fromuser,
 
 		if (submit_fork(args, envs, save_submit_errmsg) ||
 			(submit_write_message(from), submit_readrcprinterr()))
-		{
-			submit_err=1;
+		{			submit_err=1;
 			return;
 		}
 		while (j)
@@ -393,7 +392,10 @@ static void dofwd(const char *addr, const char *from, const char *fromuser,
 
 	if (!submit_err)
 	{
-	char	*p;
+		char	*p;
+
+		if (!origuser)
+			origuser="";
 
 		p=strcat(strcat(strcpy(courier_malloc(
 			strlen(addr)+strlen(origuser)+4), addr),
@@ -436,18 +438,16 @@ char	buf[BUFSIZ];
 int	waitstat;
 char	*sender=verp_getsender(ctf, ctf->receipients[rcptnum]);
 
-const	char *oreceipient;
-char	*oreceipients=0;
+const	char *oreceipient_utf8;
 
-	if ( ctf->oreceipients[rcptnum] == 0 ||
-			ctf->oreceipients[rcptnum][0])
+	if ( ctf->oreceipients_utf8[rcptnum] == 0 ||
+	     ctf->oreceipients_utf8[rcptnum][0])
 	{
-		oreceipient=ctf->oreceipients[rcptnum];
+		oreceipient_utf8=ctf->oreceipients_utf8[rcptnum];
 	}
 	else
 	{
-		oreceipient=oreceipients=dsnencodeorigaddr(
-			ctf->receipients[rcptnum]);
+		oreceipient_utf8=ctf->receipients[rcptnum];
 	}
 
 	++maxfd;
@@ -487,9 +487,15 @@ char	*oreceipients=0;
 				fwdpipe=-1;
 			}
 			else
-				fwdmsg(buf, l, sender,
-					ctf->receipients[rcptnum],
-					oreceipient);
+			{
+				const char *fwduser=
+					ctf->receipients[rcptnum];
+				char *fwduser_ace=udomainace(fwduser);
+
+				fwdmsg(buf, l, sender, fwduser_ace,
+				       oreceipient_utf8);
+				free(fwduser_ace);
+			}
 		}
 	}
 
@@ -526,10 +532,15 @@ char	*oreceipients=0;
 			submit_err=1;
 		else
 		{
-		int	c;
+			int	c;
+			const char *address=ctf->receipients[rcptnum];
+
+			char *address_ace=udomainace(address);
 
 			fprintf(submit_to, "\nDelivered-To: %s\n",
-				ctf->receipients[rcptnum]);
+				address_ace);
+			free(address_ace);
+
 			while ((c=getc(f)) != EOF)
 				putc(c, submit_to);
 			if (ferror(f) || fflush(submit_to) || fclose(submit_to))
@@ -564,14 +575,10 @@ char	*oreceipients=0;
 					"Message delivered.",
 					COMCTLFILE_DELSUCCESS, " l");
 			}
-			if (oreceipients)
-				free (oreceipients);
 			return;
 		case EX_SOFTWARE:
 			ctlfile_append_reply(ctf, rcptnum,
 					"", COMCTLFILE_DELFAIL_NOTRACK, 0);
-			if (oreceipients)
-				free (oreceipients);
 			return;
 
 
@@ -589,12 +596,8 @@ char	*oreceipients=0;
 		default:
 			ctlfile_append_reply(ctf, rcptnum,
 					"", COMCTLFILE_DELDEFERRED, 0);
-			if (oreceipients)
-				free (oreceipients);
 			return;
 		}
 
 	ctlfile_append_reply(ctf, rcptnum, "", COMCTLFILE_DELFAIL, 0);
-	if (oreceipients)
-		free (oreceipients);
 }
