@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2011 Double Precision, Inc.
+** Copyright 1998 - 2018 Double Precision, Inc.
 ** See COPYING for distribution information.
 **
 */
@@ -54,12 +54,20 @@
 #include	"maildir/maildirgetquota.h"
 #include	"maildir/maildircreate.h"
 #include	"maildir/loginexec.h"
+#include	"rfc2045/rfc2045.h"
 #include	"courierauth.h"
 
 #define POP3DLIST "courierpop3dsizelist"
+#define LISTVERSION 3
 
 extern void pop3dcapa();
 static void acctout(const char *disc);
+void rfc2045_error(const char *p)
+{
+	if (write(2, p, strlen(p)) < 0)
+		_exit(1);
+	_exit(0);
+}
 
 static const char *authaddr, *remoteip, *remoteport;
 
@@ -68,13 +76,14 @@ struct msglist {
 	char *filename;
 	int isdeleted;
 	int isnew;
+	int isutf8;
 	off_t size;
 
 	struct {
 		unsigned long uidv;
 		unsigned long n;
 	} uid;
-	} ;
+} ;
 
 static struct msglist *msglist_l;
 static struct msglist **msglist_a;
@@ -82,6 +91,8 @@ static unsigned msglist_cnt;
 
 static struct stat enomem_stat;
 static int enomem_1msg;
+int utf8_enabled;
+int savesizes=0;
 
 /*
 ** When a disk error occurs while saving an updated courierpop3dsizelist
@@ -109,7 +120,6 @@ static unsigned long bytes_sent_count=0;
 static unsigned long bytes_received_count=0;
 
 static unsigned long uidv=0;
-static int convert_v0=0;
 
 static time_t start_time;
 
@@ -120,29 +130,19 @@ static time_t start_time;
 
 static void calcsize(struct msglist *m)
 {
-FILE	*f=fopen(m->filename, "r");
-int	c, lastc;
+	FILE	*f=fopen(m->filename, "r");
+	struct rfc2045 *p=rfc2045_fromfp(f);
 
-	m->size=0;
-	if (f == 0)
-	{
-		perror("calcsize fopen");
-		return;
-	}
-	lastc='\n';
-	while ((c=getc(f)) >= 0)
-	{
-		if (c == '\n')	++m->size;
-		++m->size;
-		lastc=c;
-	}
-	if (lastc != '\n')	m->size += 2;
+	m->size=p->nlines + p->endpos;
 
-	if (ferror(f))
-	{
-		perror("calcsize ferror");
-		return;
-	}
+	clearerr(f);
+	if (m->size > 0 && fseek(f, -1, SEEK_SET) == 0 && getc(f) != '\n')
+		m->size+=2; /* We'll add an extra CRLF ourselves */
+
+	m->isutf8=0;
+	if (p->rfcviolation & RFC2045_ERR8BITHEADER)
+		m->isutf8=1;
+	rfc2045_free(p);
 	fclose(f);
 }
 
@@ -180,36 +180,31 @@ static struct msglist **readpop3dlist(unsigned long *uid)
 	struct msglist *list=NULL;
 	size_t mcnt=0;
 
-	char linebuf[1024];
+	char linebuf[2048];
 
 	FILE *fp=openpop3dlist();
 
 	size_t i;
 	int vernum=0;
+	unsigned long size;
 
 	uidv=time(NULL);
 
-	convert_v0=0;
-
 	if (fp == NULL ||
 	    fgets(linebuf, sizeof(linebuf)-1, fp) == NULL ||
 	    linebuf[0] != '/' || sscanf(linebuf+1, "%d %lu %lu", &vernum,
 					uid, &uidv)
-	    < 2 || (vernum != 1 && vernum != 2))
+	    < 2 || (vernum != LISTVERSION &&
+		    vernum != (LISTVERSION-1)))
 	{
-		if (fp == NULL)
-			convert_v0=1;
-
-		if (vernum == 0 && fp && fseek(fp, 0L, SEEK_SET) >= 0)
-		{
-			/* Old version 0 format courierpop3dsizelist file */
-		}
-		else
-		{
-			if (fp)
-				fclose(fp);
-			fp=NULL;
-		}
+		if (fp)
+			fclose(fp);
+		fp=NULL;
+	}
+	else
+	{
+		if (vernum != LISTVERSION)
+			savesizes=1;
 	}
 
 	if (fp)
@@ -223,21 +218,15 @@ static struct msglist **readpop3dlist(unsigned long *uid)
 
 		while ((ch=getc(fp)) != EOF)
 		{
-			unsigned long sz;
-
 			if (ch != '\n')
 			{
-				if (n < sizeof(linebuf)-3)
+				if (n < sizeof(linebuf)-20)
 					linebuf[n++]=ch;
 				continue;
 			}
 			linebuf[n]=0;
 			n=0;
 
-			if (vernum == 0)
-				strcat(linebuf, " 0");
-			/* Convert version 0 to version 1 format - PRESTO! */
-
 			if ((p=strrchr(linebuf, ' ')) == NULL)
 				continue;
 			*p=0;
@@ -263,18 +252,21 @@ static struct msglist **readpop3dlist(unsigned long *uid)
 				exit(1);
 			}
 
-			switch (sscanf(p, "%lu %lu:%lu", &sz,
-				       &m->uid.n, &m->uid.uidv)) {
-			case 2:
-				m->uid.uidv=0;
-				/* FALLTHROUGH */
-			case 3:
-				m->size=sz;
+			// Converting from LISTVERSION 2, assuming no UTF-8.
+			// We have extra room at the end.
+			strcat(p, ":0");
+
+			if (sscanf(p, "%lu %lu:%lu:%d", &size,
+				   &m->uid.n, &m->uid.uidv,
+				   &m->isutf8) == 4)
+			{
+				m->size=size;
 				m->next=list;
 				list=m;
 				++mcnt;
-				break;
-			default:
+			}
+			else
+			{
 				free(m->filename);
 				free(m);
 			}
@@ -316,7 +308,7 @@ static int savepop3dlist(struct msglist **a, size_t cnt,
 		return -1;
 	}
 
-	fprintf(fp, "/2 %lu %lu\n", uid, uidv);
+	fprintf(fp, "/%d %lu %lu\n", LISTVERSION, uid, uidv);
 
 	for (i=0; i<cnt; i++)
 	{
@@ -326,8 +318,8 @@ static int savepop3dlist(struct msglist **a, size_t cnt,
 		if ((q=strrchr(p, '/')) != NULL)
 			p=q+1;
 
-		fprintf(fp, "%s %lu %lu:%lu\n", p, (unsigned long)a[i]->size,
-			a[i]->uid.n, a[i]->uid.uidv);
+		fprintf(fp, "%s %lu %lu:%lu:%d\n", p, (unsigned long)a[i]->size,
+			a[i]->uid.n, a[i]->uid.uidv, a[i]->isutf8);
 	}
 
 	if (fflush(fp) || ferror(fp))
@@ -411,7 +403,7 @@ static int cmpmsgs(const void *a, const void *b)
 		++bp;
 	else
 		bp=bname;
- 
+
 	na=atol(ap);
 	nb=atol(bp);
 
@@ -439,7 +431,6 @@ static void sortmsgs()
 	size_t i, n;
 	struct msglist *m;
 	struct msglist **prev_list;
-	int savesizes=0;
 
 	unsigned long nextuid;
 
@@ -480,6 +471,7 @@ static void sortmsgs()
 		{
 			msglist_a[i]->size=prev_list[n]->size;
 			msglist_a[i]->uid=prev_list[n]->uid;
+			msglist_a[i]->isutf8=prev_list[n]->isutf8;
 			n++;
 		}
 		else
@@ -487,12 +479,10 @@ static void sortmsgs()
 			msglist_a[i]->uid.n=nextuid++;
 			msglist_a[i]->uid.uidv=uidv;
 			msglist_a[i]->isnew=1;
-			if (convert_v0)
-				msglist_a[i]->uid.n=0;
 
 			calcsize(msglist_a[i]);
 			savesizes=1;
-		}      
+		}
 	}
 
 	if (prev_list[n])
@@ -568,9 +558,10 @@ static void cleanup()
 			}
 
 			if (log_deletions)
-				fprintf(stderr, "INFO: DELETED, user=%s, ip=[%s], filename=%s\n",
+				fprintf(stderr, "INFO: DELETED, user=%s, ip=[%s], port=[%s], filename=%s\n",
 					getenv("AUTHENTICATED"),
-					getenv("TCPREMOTEIP"),
+					remoteip,
+					remoteport,
 					msglist_a[i]->filename);
 
 			if (unlink(msglist_a[i]->filename))
@@ -660,14 +651,89 @@ char	buf[NUMBUFSIZE];
 
 /* RETR and TOP POP3 commands */
 
+#define MIMEWRAPTXT							\
+	"From: Mail Delivery Subsystem <postmaster>\n"			\
+	"Subject: Cannot display Unicode content\n"			\
+	"Mime-Version: 1.0\n"						\
+	"Content-Type: multipart/mixed; boundary=\"%s\"\n"		\
+	"\n\n\n--%s\n"							\
+	"Content-Type: text/plain\n\n"					\
+	"This E-mail message was determined to be Unicode-formatted\n"	\
+	"but your E-mail reader does not support Unicode E-mail.\n\n"	\
+	"Please use an E-mail reader that supports POP3 with UTF-8\n"	\
+	"(see https://tools.ietf.org/html/rfc6856.html).\n\n"		\
+	"This can also happen when the sender's E-mail program does not\n" \
+	"correctly format the sent message.\n\n"			\
+	"The original message is included as a separate attachment\n"	\
+	"so that it can be downloaded manually.\n\n"			\
+	"--%s\n"							\
+	"Content-Type: %s\n\n"
+
+struct retr_source {
+	char *wrapheader;
+	FILE *f;
+	char *wrapfooter;
+};
+
+static int get_retr_source(struct retr_source *s)
+{
+	int c=(unsigned char)*s->wrapheader;
+
+	if (c)
+	{
+		++s->wrapheader;
+		return c;
+	}
+
+	if (s->f)
+	{
+		int c=getc(s->f);
+
+		if (c >= 0)
+			return c;
+		s->f=0;
+	}
+
+	if (*s->wrapfooter)
+	{
+		int c=(unsigned char)*s->wrapfooter;
+
+		++s->wrapfooter;
+		return c;
+	}
+	return -1;
+}
+
 static void do_retr(unsigned i, unsigned *lptr)
 {
-FILE	*f=fopen(msglist_a[i]->filename, "r");
-char	*p;
-int	c, lastc='\n';
-int	inheader=1;
-char	buf[NUMBUFSIZE];
-unsigned long *cntr;
+	FILE	*f;
+	char	*p;
+	int	c, lastc='\n';
+	int	inheader=1;
+	char	buf[NUMBUFSIZE];
+	unsigned long *cntr;
+	char boundary[256];
+	char wrapheader[sizeof(MIMEWRAPTXT)+1024];
+	char wrapfooter[512];
+	const char *mime_message_type=getenv("MIME_UNICODE_MESSAGE_TYPE");
+	struct retr_source rs;
+
+	wrapheader[0]=0;
+	wrapfooter[0]=0;
+
+	if (!mime_message_type)
+		mime_message_type="message/global";
+
+	if (msglist_a[i]->isutf8 && !utf8_enabled)
+	{
+		sprintf(boundary, "=_%d-%d", (int)getpid(), (int)time(NULL));
+
+		sprintf(wrapheader, MIMEWRAPTXT, boundary, boundary, boundary,
+			mime_message_type);
+		sprintf(wrapfooter, "\n--%s--\n", boundary);
+	}
+
+	f=fopen(msglist_a[i]->filename, "r");
 
 	if (!f)
 	{
@@ -676,13 +742,20 @@ unsigned long *cntr;
 		return;
 	}
 	printed(printf( (lptr ? "+OK headers follow.\r\n":"+OK %s octets follow.\r\n"),
-			libmail_str_off_t(msglist_a[i]->size, buf)));
+			libmail_str_off_t(msglist_a[i]->size +
+					  strlen(wrapheader) +
+					  strlen(wrapfooter),
+					  buf)));
 
 	cntr= &retr_count;
 	if (lptr)
 		cntr= &top_count;
 
-	for (lastc=0; (c=getc(f)) >= 0; lastc=c)
+	rs.wrapheader=wrapheader;
+	rs.f=f;
+	rs.wrapfooter=wrapfooter;
+
+	for (lastc=0; (c=get_retr_source(&rs)) >= 0; lastc=c)
 	{
 		if (lastc == '\n')
 		{
@@ -991,6 +1064,15 @@ int	c;
 			continue;
 		}
 
+              if (strcmp(p, "UTF8") == 0)
+              {
+                      /* XXX workaround for MS Outlook */
+                      utf8_enabled=1;
+                      printed(printf("+OK UTF8 enabled\r\n"));
+                      fflush(stdout);
+                      continue;
+              }
+
 error:
 		printed(printf("-ERR Invalid command.\r\n"));
 		fflush(stdout);
@@ -1045,6 +1127,14 @@ char	*p;
 			authaddr="nobody";
 	}
 
+	utf8_enabled=1; /* Until proven otherwise */
+
+	{
+		const char *utf8=getenv("UTF8");
+
+		if (utf8)
+			utf8_enabled=atoi(utf8);
+	}
 	if ((remoteip=getenv("TCPREMOTEIP")) == NULL)
 		remoteip="127.0.0.1";
 
@@ -1078,7 +1168,7 @@ char	*p;
 		fflush(stdout);
 		exit(1);
 	}
-	
+
 	maildir_loginexec();
 
 	if (auth_getoptionenvint("disablepop3"))
