@@ -13,11 +13,11 @@
 struct htmlfilter_info;
 
 extern struct htmlfilter_info
-*htmlfilter_alloc(void (*)(const unicode_char *, size_t, void *), void *);
+*htmlfilter_alloc(void (*)(const char32_t *, size_t, void *), void *);
 extern void htmlfilter_free(struct htmlfilter_info *);
 
 extern void htmlfilter(struct htmlfilter_info *,
-		       const unicode_char *, size_t);
+		       const char32_t *, size_t);
 
 extern void htmlfilter_set_contentbase(struct htmlfilter_info *,
 				   const char *);
