@@ -1,5 +1,5 @@
 /*
-** Copyright 2007-2013 Double Precision, Inc.
+** Copyright 2007-2019 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 #include	"config.h"
@@ -13,12 +13,12 @@
 #include	<gnutls/extra.h>
 #endif
 #include	<gnutls/x509.h>
-#include	<gnutls/openpgp.h>
 #include	<stdio.h>
 #include	<string.h>
 #include	<stdlib.h>
 #include	<ctype.h>
 #include	<netdb.h>
+#include	<idna.h>
 #if HAVE_DIRENT_H
 #include <dirent.h>
 #define NAMLEN(dirent) strlen((dirent)->d_name)
@@ -127,6 +127,7 @@ struct ssl_context_t {
 	const char *priority_list;
 
 	char *certfile;
+	char *keyfile;
 	char *dhfile;
 
 	char *trustcerts;
@@ -146,9 +147,6 @@ struct ssl_handle_t {
 	gnutls_session_t session;
 
 	gnutls_x509_privkey_t x509_key;
-
-	gnutls_openpgp_crt_t pgp_crt;
-	gnutls_openpgp_privkey_t pgp_key;
 };
 
 static void nonsslerror(struct tls_info *info, const char *pfix)
@@ -271,6 +269,17 @@ ssl_context tls_create(int isserver, const struct tls_info *info)
 			free(certfile);
 	}
 
+	if ((certfile=strdup(safe_getenv(p, "TLS_PRIVATE_KEYFILE", ""))) != NULL &&
+	    *certfile)
+	{
+		p->keyfile=certfile;
+	}
+	else
+	{
+		if (certfile)
+			free(certfile);
+	}
+
 	switch (*safe_getenv(p, "TLS_VERIFYPEER", "P")) {
 	case 'n':
 	case 'N':
@@ -324,6 +333,8 @@ void tls_destroy(ssl_context p)
 {
 	if (p->certfile)
 		free(p->certfile);
+	if (p->keyfile)
+		free(p->keyfile);
 	if (p->dhfile)
 		free(p->dhfile);
 	if (p->trustcerts)
@@ -367,7 +378,7 @@ static int read_cert_dir(const char *cert_dir,
 
 		strcat(strcat(strcpy(buf, cert_dir), "/"), de->d_name);
 
-		if (lstat(buf, &stat_buf) < 0 || !S_ISREG(stat_buf.st_mode))
+		if (stat(buf, &stat_buf) < 0 || !S_ISREG(stat_buf.st_mode))
 		{
 			free(buf);
 			continue;
@@ -475,16 +486,7 @@ static void tls_free_session_keys(ssl_handle ssl)
 	if (ssl->x509_key)
 		gnutls_x509_privkey_deinit(ssl->x509_key);
 
-	if (ssl->pgp_crt)
-		gnutls_openpgp_key_deinit(ssl->pgp_crt);
-
-	if (ssl->pgp_key)
-		gnutls_openpgp_privkey_deinit(ssl->pgp_key);
-
 	ssl->x509_key=NULL;
-	ssl->pgp_crt=NULL;
-	ssl->pgp_key=NULL;
-
 }
 
 static void tls_free_session(ssl_handle ssl)
@@ -537,13 +539,52 @@ static int chk_error(int rc, ssl_handle ssl, int fd, fd_set *r, fd_set *w,
 
 	if (result_rc)
 	{
-		(*ssl->info_cpy.tls_err_msg)(gnutls_strerror(rc),
+		const char *errmsg=gnutls_strerror(rc);
+
+#ifdef GNUTLS_E_PREMATURE_TERMINATION
+		if (rc == GNUTLS_E_PREMATURE_TERMINATION)
+		{
+			errmsg="DEBUG: Unexpected SSL connection shutdown.";
+		}
+#endif
+		(*ssl->info_cpy.tls_err_msg)(errmsg,
 					     ssl->info_cpy.app_data);
 		*result_rc= -1;
 	}
 	return 0;
 }
 
+static int name_check(ssl_handle ssl,
+		      gnutls_x509_crt_t cert)
+{
+	char *idn_domain;
+	const char *p;
+	int rc;
+
+	if (idna_to_unicode_8z8z(ssl->info_cpy.peer_verify_domain,
+				 &idn_domain, 0) != IDNA_SUCCESS)
+		idn_domain=0;
+
+	p=idn_domain ? idn_domain:ssl->info_cpy.peer_verify_domain;
+
+	rc=gnutls_x509_crt_check_hostname(cert, p);
+
+	if (idn_domain)
+	{
+		free(idn_domain);
+
+		/* Now try the ACE-encoded hostname for a filename. */
+
+		if (rc == 0)
+		{
+			p=ssl->info_cpy.peer_verify_domain;
+			rc=gnutls_x509_crt_check_hostname(cert, p);
+		}
+	}
+	return rc;
+}
+
+
 static int verify_client(ssl_handle ssl, int fd)
 {
 	unsigned int status;
@@ -637,10 +678,7 @@ static int verify_client(ssl_handle ssl, int fd)
 
 		if (ssl->info_cpy.peer_verify_domain &&
 		    *ssl->info_cpy.peer_verify_domain &&
-		    !gnutls_x509_crt_check_hostname(cert,
-						    ssl->info_cpy
-						    .peer_verify_domain
-						    ))
+		    !name_check(ssl, cert))
 		{
 			char hostname[256];
 			size_t hostname_size=sizeof(hostname);
@@ -672,94 +710,6 @@ static int verify_client(ssl_handle ssl, int fd)
 
 		gnutls_x509_crt_deinit (cert);
 	}
-	else if (gnutls_certificate_type_get(ssl->session)==GNUTLS_CRT_OPENPGP)
-	{
-		gnutls_openpgp_crt_t cert;
-
-		if (gnutls_openpgp_key_init(&cert) < 0)
-		{
-			(*ssl->info_cpy.tls_err_msg)
-				("Error initializing certificate",
-				 ssl->info_cpy.app_data);
-			gnutls_openpgp_key_deinit(cert);
-			return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
-		}
-
-		if (gnutls_openpgp_key_import(cert, &cert_list[0],
-					      GNUTLS_OPENPGP_FMT_RAW) < 0)
-		{
-			(*ssl->info_cpy.tls_err_msg)
-				("Error parsing certificate",
-				 ssl->info_cpy.app_data);
-			gnutls_openpgp_key_deinit (cert);
-			return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
-		}
-
-		if (gnutls_openpgp_key_get_creation_time(cert) > time(NULL))
-		{
-			(*ssl->info_cpy.tls_err_msg)
-				("Certificate not activated",
-				 ssl->info_cpy.app_data);
-			gnutls_openpgp_key_deinit (cert);
-			return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
-		}
-
-		if (gnutls_openpgp_key_get_expiration_time(cert) < time(NULL))
-		{
-			(*ssl->info_cpy.tls_err_msg)
-				("Expired certificate",
-				 ssl->info_cpy.app_data);
-			gnutls_openpgp_key_deinit (cert);
-			return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
-		}
-
-		if (ssl->info_cpy.peer_verify_domain &&
-		    *ssl->info_cpy.peer_verify_domain &&
-		    !gnutls_openpgp_key_check_hostname(cert,
-						       ssl->info_cpy
-						       .peer_verify_domain))
-
-		{
-			char *hostname;
-			size_t hostnamesiz=0;
-			const char *errmsg_txt=
-				"Certificate owner mismatch: ";
-			char *errmsg_buf;
-
-			gnutls_openpgp_key_get_name(cert, 0, NULL,
-						    &hostnamesiz);
-
-			hostname=malloc(hostnamesiz);
-
-			if (hostname)
-			{
-				*hostname=0;
-				gnutls_openpgp_key_get_name(cert,
-							    0, hostname,
-							    &hostnamesiz);
-			}
-
-			errmsg_buf=malloc(strlen(errmsg_txt)+
-					  strlen(hostname ?
-						 hostname:"")+100);
-
-			if (errmsg_buf)
-				strcat(strcpy(errmsg_buf, errmsg_txt),
-					       hostname ?
-					       hostname:"(unknown)");
-
-			(*ssl->info_cpy.tls_err_msg)
-				(errmsg_buf ? errmsg_buf:strerror(errno),
-				 ssl->info_cpy.app_data);
-			if (errmsg_buf)
-				free(errmsg_buf);
-			if (hostname)
-				free(hostname);
-			gnutls_openpgp_key_deinit (cert);
-			return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
-		}
-		gnutls_openpgp_key_deinit (cert);
-	}
 	else
 	{
 		(*ssl->info_cpy.tls_err_msg)
@@ -796,10 +746,10 @@ static int dohandshake(ssl_handle ssl, int fd, fd_set *r, fd_set *w)
 	return rc;
 }
 
-static char *check_cert(const char *filename,
-			gnutls_certificate_type_t cert_type,
-			const char *req_dn,
-			int isvirtual)
+static char *check_cert_unicode(const char *filename,
+				gnutls_certificate_type_t cert_type,
+				const char *req_dn,
+				int isvirtual)
 {
 	if (!filename || !*filename)
 		return NULL;
@@ -813,9 +763,6 @@ static char *check_cert(const char *filename,
 
 		strcat(strcat(strcpy(p, filename), "."), req_dn);
 
-		if (cert_type == GNUTLS_CRT_OPENPGP)
-			strcat(p, ".pgp");
-
 		if (access(p, R_OK) == 0)
 			return p;
 
@@ -829,6 +776,12 @@ static char *check_cert(const char *filename,
 		++req_dn;
 	}
 
+	/*
+	** We're called with a hostname first. Don't check the defualt
+	** filename, we'll be called again with an IP address
+	*/
+
+	if (!isvirtual)
 	{
 		char *p=malloc(strlen(filename)+10);
 
@@ -837,9 +790,6 @@ static char *check_cert(const char *filename,
 
 		strcpy(p, filename);
 
-		if (cert_type == GNUTLS_CRT_OPENPGP)
-			strcat(p, ".pgp");
-
 		if (access(p, R_OK) == 0)
 			return p;
 
@@ -848,6 +798,41 @@ static char *check_cert(const char *filename,
 	return NULL;
 }
 
+static char *check_cert(const char *filename,
+			gnutls_certificate_type_t cert_type,
+			const char *req_dn,
+			int isvirtual)
+{
+	if (isvirtual)
+	{
+		char *p;
+		char *retfile;
+
+		if (idna_to_ascii_8z(req_dn, &p, 0) != IDNA_SUCCESS)
+			p=0;
+
+		if (p)
+		{
+			retfile=check_cert_unicode(filename, cert_type, p,
+						    isvirtual);
+			free(p);
+
+			if (retfile)
+				return retfile;
+		}
+	}
+
+	return check_cert_unicode(filename, cert_type, req_dn, isvirtual);
+}
+
+static char *check_key(const char *filename,
+			gnutls_certificate_type_t cert_type,
+			const char *req_dn,
+			int isvirtual)
+{
+	return check_cert(filename, cert_type, req_dn, isvirtual);
+}
+
 static int read_file(const char *file,
 		     gnutls_datum_t *filebuf)
 {
@@ -889,10 +874,12 @@ static void release_file(gnutls_datum_t *filebuf)
 static int set_cert(ssl_handle ssl,
 		    gnutls_session_t session,
 		    gnutls_retr2_st *st,
-		    const char *certfilename)
+		    const char *certfilename,
+		    const char *keyfilename)
 {
 	int rc;
 	gnutls_datum_t filebuf;
+	gnutls_datum_t keyfilebuf;
 	unsigned int cert_cnt;
 
 	st->ncerts=0;
@@ -902,15 +889,34 @@ static int set_cert(ssl_handle ssl,
 	if ((rc=read_file(certfilename, &filebuf)) < 0)
 		return rc;
 
+	if (keyfilename)
+	{
+		if ((rc=read_file(keyfilename, &keyfilebuf)) < 0)
+		{
+			release_file(&filebuf);
+			return rc;
+		}
+	}
+
 	switch (st->cert_type) {
 	case GNUTLS_CRT_X509:
 
 		cert_cnt=0;
 
-		if ((rc=gnutls_x509_privkey_init(&ssl->x509_key)) < 0 ||
-		    (rc=gnutls_x509_privkey_import(ssl->x509_key, &filebuf,
-						   GNUTLS_X509_FMT_PEM)) < 0)
-			break;
+		if (keyfilename)
+		{
+			if ((rc=gnutls_x509_privkey_init(&ssl->x509_key)) < 0 ||
+			    (rc=gnutls_x509_privkey_import(ssl->x509_key, &keyfilebuf,
+							   GNUTLS_X509_FMT_PEM)) < 0)
+				break;
+		}
+		else
+		{
+			if ((rc=gnutls_x509_privkey_init(&ssl->x509_key)) < 0 ||
+			    (rc=gnutls_x509_privkey_import(ssl->x509_key, &filebuf,
+							   GNUTLS_X509_FMT_PEM)) < 0)
+				break;
+		}
 
 		rc=gnutls_x509_crt_list_import(NULL, &cert_cnt,
 					       &filebuf,
@@ -939,26 +945,14 @@ static int set_cert(ssl_handle ssl,
 		ssl->x509_key=0;
 		st->deinit_all=1;
 
-		break;
-	case GNUTLS_CRT_OPENPGP:
-		if ((rc=gnutls_openpgp_key_init(&ssl->pgp_crt)) < 0 ||
-		    (rc=gnutls_openpgp_privkey_init(&ssl->pgp_key)) < 0 ||
-		    (rc=gnutls_openpgp_key_import(ssl->pgp_crt, &filebuf,
-						  GNUTLS_OPENPGP_FMT_BASE64))
-		    < 0 ||
-		    (rc=gnutls_openpgp_privkey_import(ssl->pgp_key, &filebuf,
-						      GNUTLS_OPENPGP_FMT_BASE64,
-						      NULL, 0)) < 0)
-			break;
-		st->cert.pgp=ssl->pgp_crt;
-		st->ncerts=1;
-		st->key.pgp=ssl->pgp_key;
 		break;
 	default:
 		break;
 	}
 
 	release_file(&filebuf);
+	if (keyfilename)
+		release_file(&keyfilebuf);
 	return 0;
 }
 
@@ -975,6 +969,7 @@ static int get_server_cert(gnutls_session_t session,
 	size_t vhost_size;
 	unsigned int type=GNUTLS_NAME_DNS;
 	char *certfilename=NULL;
+	char *keyfilename=NULL;
 	int rc;
 
 	st->cert_type=gnutls_certificate_type_get(session);
@@ -1000,35 +995,64 @@ static int get_server_cert(gnutls_session_t session,
 	     ++vhost_idx)
 	{
 		char *p;
+		char *utf8;
+		char *namebuf;
 
-		for (p=vhost_buf; *p; p++)
+		/* Convert to UTF8 */
+		if (idna_to_unicode_8z8z(vhost_buf, &utf8, 0)
+		    != IDNA_SUCCESS)
+			utf8=0;
+
+		namebuf=utf8 ? utf8:vhost_buf;
+
+		for (p=namebuf; *p; p++)
 			if (*p == '/')
 				*p='.'; /* Script kiddie check */
 
 		if (ssl->ctx->certfile)
 			certfilename=check_cert(ssl->ctx->certfile,
 						st->cert_type,
-						vhost_buf, 1);
+						namebuf, 1);
 
+		if (ssl->ctx->keyfile)
+			keyfilename=check_key(ssl->ctx->keyfile,
+						st->cert_type,
+						namebuf, 1);
+
+		if (utf8)
+			free(utf8);
 		if (certfilename)
 			break;
 	}
 
 	if (!certfilename)
 	{
+		const char *ip=
+			safe_getenv(ssl->ctx,
+				    "TCPLOCALIP", "");
+
+		if (strncmp(ip, "::ffff:", 7) == 0 && strchr(ip, '.'))
+			ip += 7;
+
 		if (ssl->ctx->certfile)
 			certfilename=check_cert(ssl->ctx->certfile,
-						st->cert_type,
-						safe_getenv(ssl->ctx,
-							    "TCPLOCALIP", ""),
-						0);
+						st->cert_type, ip, 0);
+		if (ssl->ctx->keyfile)
+			keyfilename=check_key(ssl->ctx->keyfile,
+					      st->cert_type, ip, 0);
 	}
 
 	if (!certfilename)
+	{
+		if (ssl->ctx->certfile)
+			nonsslerror(&ssl->info_cpy, ssl->ctx->certfile);
 		return 0;
+	}
 
-	rc=set_cert(ssl, session, st, certfilename);
+	rc=set_cert(ssl, session, st, certfilename, keyfilename);
 	free(certfilename);
+	if (keyfilename)
+		free(keyfilename);
 	return rc;
 }
 
@@ -1181,6 +1205,7 @@ static int get_client_cert(gnutls_session_t session,
 	ssl_handle ssl=(ssl_handle)gnutls_session_get_ptr(session);
 	int rc;
 	char *certfilename=NULL;
+	char *keyfilename=NULL;
 
 	rc= 0;
 	st->cert_type=gnutls_certificate_type_get(session);
@@ -1189,12 +1214,16 @@ static int get_client_cert(gnutls_session_t session,
 		certfilename=check_cert(ssl->ctx->certfile,
 					st->cert_type, "", 0);
 
+	if (ssl->ctx->keyfile)
+		keyfilename=check_key(ssl->ctx->keyfile,
+					st->cert_type, "", 0);
+
 	st->ncerts=0;
 	st->deinit_all=0;
 
 	if (certfilename)
 	{
-		rc=set_cert(ssl, session, st, certfilename);
+		rc=set_cert(ssl, session, st, certfilename, keyfilename);
 		free(certfilename);
 	}
 	else
@@ -1204,6 +1233,10 @@ static int get_client_cert(gnutls_session_t session,
 		if (rc > 0)
 			rc=0;
 	}
+
+	if (keyfilename)
+		free(keyfilename);
+
 	return rc;
 }
 
@@ -1334,6 +1367,26 @@ static gnutls_datum_t db_retrieve_func(void *dummy, gnutls_datum_t key)
 	return drs.ret;
 }
 
+static int name_set(ssl_handle ssl, ssl_context ctx)
+{
+	char *idn_domain;
+	const char *p;
+	int rc;
+
+	if (idna_to_unicode_8z8z(ctx->info_cpy.peer_verify_domain,
+				 &idn_domain, 0) != IDNA_SUCCESS)
+		idn_domain=0;
+
+	p=idn_domain ? idn_domain:ctx->info_cpy.peer_verify_domain;
+
+	rc=gnutls_server_name_set(ssl->session, GNUTLS_NAME_DNS,
+				  p, strlen(p));
+
+	if (idn_domain)
+		free(idn_domain);
+	return rc;
+}
+
 ssl_handle tls_connect(ssl_context ctx, int fd)
 {
 	ssl_handle ssl=malloc(sizeof(struct ssl_handle_t));
@@ -1475,10 +1528,7 @@ RT |
 				   ssl->xcred) < 0 ||
 
 	    (ctx->info_cpy.peer_verify_domain &&
-	     gnutls_server_name_set(ssl->session, GNUTLS_NAME_DNS,
-				    ctx->info_cpy.peer_verify_domain,
-				    strlen(ctx->info_cpy.peer_verify_domain))
-	     < 0)
+	     name_set(ssl, ctx) < 0)
 	    )
 	{
 		tls_free_session(ssl);
@@ -1670,10 +1720,12 @@ int tls_connecting(ssl_handle ssl)
 }
 
 static const char *dump_dn(gnutls_x509_crt_t cert,
-			   int (*get_dn_func)(gnutls_x509_crt_t cert, int indx,
+			   int (*get_dn_func)(gnutls_x509_crt_t cert,
+					      unsigned indx,
 					      void *oid, size_t * sizeof_oid),
 			   int (*get_dnval_func)(gnutls_x509_crt_t cert,
-						 const char *oid, int indx,
+						 const char *oid,
+						 unsigned indx,
 						 unsigned int raw_flag,
 						 void *buf, size_t *sizeof_buf),
 			   void (*dump_func)(const char *, int cnt, void *),
@@ -1867,8 +1919,6 @@ static void dump_cipher_name(gnutls_session_t session,
 		(*dump_func)(cipher_name, -1, dump_arg);
 	else
 	{
-		gnutls_compression_method_t comp;
-
 		(*dump_func)(gnutls_kx_get_name(kx_algo), -1, dump_arg);
 
 		(*dump_func)("-", 1, dump_arg);
@@ -1879,14 +1929,6 @@ static void dump_cipher_name(gnutls_session_t session,
 		(*dump_func)(gnutls_cipher_get_name(cipher_algo), -1,
 			     dump_arg);
 
-		if ((comp=gnutls_compression_get(session))
-		    != GNUTLS_COMP_NULL)
-		{
-			(*dump_func)("/", 1, dump_arg);
-			(*dump_func)(gnutls_compression_get_name(comp),
-				     -1, dump_arg);
-		}
-
 		(*dump_func)("-", 1, dump_arg);
 		(*dump_func)(gnutls_mac_get_name(gnutls_mac_get(session)),
 			     -1, dump_arg);
@@ -1923,6 +1965,8 @@ static void tls_dump_connection_info_x509(ssl_handle ssl,
 		{
 			time_t notbefore;
 			time_t notafter;
+			char buffer[256];
+			unsigned j;
 
 			(*dump_func)("Subject:\n", -1, dump_arg);
 
@@ -1932,6 +1976,29 @@ static void tls_dump_connection_info_x509(ssl_handle ssl,
 				dump_func, dump_arg);
 			(*dump_func)("\n", 1, dump_arg);
 
+			for (j=0; ; ++j)
+			{
+				size_t s=sizeof(buffer);
+				enum gnutls_x509_subject_alt_name_t t;
+
+				t=gnutls_x509_crt_get_subject_alt_name
+					(cert[i], j, buffer, &s, 0);
+
+				if (t == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE)
+					break;
+
+				if (t != GNUTLS_SAN_DNSNAME)
+					continue;
+
+				if (s == sizeof(buffer))
+					--s; /* The API is not clear */
+
+				buffer[s]=0;
+				(*dump_func)("Subject-Alt-Name-DNS: ", -1,
+					     dump_arg);
+				(*dump_func)(buffer, -1, dump_arg);
+				(*dump_func)("\n", -1, dump_arg);
+			}
 
 #if 0
 			(*dump_func)("Issuer:\n", -1, dump_arg);
